@@ -4,19 +4,35 @@ import json
 import logging
 import ssl
 import http.client
+import time
+import math
+import random
 from datetime import datetime
 from tempfile import NamedTemporaryFile
 from urllib import request
 
 from cinp.common import URI
 
-__CLIENT_VERSION__ = '0.19.1'
-__CINP_VERSION__ = '0.9'
+__CLIENT_VERSION__ = '1.0.0'
+__CINP_VERSION__ = '1.0'
 
 __all__ = [ 'Timeout', 'ResponseError', 'DetailedInvalidRequest',
             'InvalidRequest', 'InvalidSession', 'NotAuthorized',
             'NotFound', 'ServerError', 'CInP' ]
 
+DELAY_MULTIPLIER = 15
+# delay of 15 results in a delay of:
+# min delay = 0, 10, 16, 20, 24, 26, 29, 31, 32, 34, 35, 37, 38, 39, 40 ....
+# max delay = 0, 20, 32, 40, 48, 52, 58, 62, 64, 68, 70, 74, 76, 78, 80 .....
+
+DEFAULT_RETRY_ON = [
+                     'Timeout:None',
+                     'ResponseError:[Errno 101] Network is unreachable',
+                     'ResponseError:[Errno 104] Connection reset by peer',
+                     'ResponseError:Connection reset by peer',
+                     'ResponseError:[Errno 111] Connection refused'
+                  ]
+
 
 class Timeout( Exception ):
   pass
@@ -32,7 +48,7 @@ class InvalidRequest( Exception ):
 
 class DetailedInvalidRequest( InvalidRequest ):
   def __init__( self, data ):
-    for name in ( 'class', 'error', 'message' ):
+    for name in ( 'exception', 'error', 'message' ):
       setattr( self, name, None )  # this makes sure it is at least set
       try:
         setattr( self, name, data[ name ] )
@@ -60,9 +76,24 @@ class ServerError( Exception ):
   pass
 
 
+class RetryableException( Exception ):
+  def __init__( self, exception, reason=None ):
+    self.exception = exception
+    self.reason = reason
+
+
+def _backOffDelay( count ):
+  if count < 1:  # math.log dosen't do so well below 1
+    count = 1
+
+  factor = int( DELAY_MULTIPLIER * math.log( count ) )
+  return int( factor + ( random.random() * factor ) )
+
+
 class CInP():
-  def __init__( self, host, root_path, proxy=None, verify_ssl=True ):
+  def __init__( self, host, root_path, proxy=None, verify_ssl=True, retry_event=None ):  # retry_event should be an Event Object, use to cancel retry loops, if the event get's set the retry loop will throw the most recent Exception it ignored
     super().__init__()
+    self.retry_event = retry_event
 
     if not host.startswith( ( 'http:', 'https:' ) ):
       raise ValueError( 'hostname must start with http(s):' )
@@ -137,18 +168,47 @@ class CInP():
     if verb in ( 'GET', 'LIST', 'UPDATE', 'CREATE', 'DELETE', 'CALL' ) and not model:
       raise InvalidRequest( 'Verb "{0}" requires model'.format( verb ) )
 
-  def _request( self, verb, uri, data=None, header_map=None, timeout=30 ):  # TODO: implement retry
+  def _request( self, verb, uri, data=None, header_map=None, timeout=30, retry_count=0, retry_on=None ):
+    _retry_on = retry_on or DEFAULT_RETRY_ON
+
+    last_exception = None
+    for retry in range( 0, retry_count + 1 ):
+      if retry > 0:
+        logging.debug( 'cinp: retry "{0}" of "{1}" for request to "{2}"'.format( retry, retry_count, uri ) )
+        if self.retry_event is not None:
+          if self.retry_event.is_set():
+            raise last_exception
+
+          self.retry_event.wait( _backOffDelay( retry ) )
+
+        else:
+          time.sleep( _backOffDelay( retry ) )
+
+      try:
+        return self.__request( verb, uri, data, header_map, timeout )
+      except RetryableException as e:
+        signature = '{0}:{1}'.format( e.exception.__class__.__name__, e.reason )
+        if signature not in _retry_on:
+          logging.debug( 'cinp: exception signature "{0}" not in the retry_on list'.format( signature ) )
+          raise e.exception
+
+        logging.debug( 'cinp: got exception "{0}", ignoring...'.format( e ) )
+        last_exception = e.exception
+
+    raise last_exception
+
+  def __request( self, verb, uri, data=None, header_map=None, timeout=30 ):
     logging.debug( 'cinp: making "{0}" request to "{1}"'.format( verb, uri ) )
     if header_map is None:
       header_map = {}
 
-    if verb == 'UPLOAD':  # not a CINP verb, just using it to bypass some checking here in _request
+    if verb == 'UPLOAD':  # not a CINP verb, just using it to bypass some checking here in __request
       header_map[ 'Content-Type' ] = 'application/octet-stream'
       if not hasattr( data, 'read' ):
         raise InvalidRequest( 'data must be an readable stream' )
       verb = 'POST'  # not to be handled by CInP on the other end, but by a file upload handler
 
-    elif verb == 'RAWGET':  # not a CINP verb, just using it to bypass some checking here in _request
+    elif verb == 'RAWGET':  # not a CINP verb, just using it to bypass some checking here in __request
       verb = 'GET'
 
     else:
@@ -165,19 +225,19 @@ class CInP():
       resp = self.opener.open( req, timeout=timeout )
 
     except request.HTTPError as e:
-      raise ResponseError( 'HTTPError "{0}"'.format( e ) )
+      raise RetryableException( ResponseError( 'HTTPError "{0}"'.format( e ) ), reason=e.reason )
 
     except request.URLError as e:
       if isinstance( e.reason, socket.timeout ):
-        raise Timeout( 'Request Timeout after {0} seconds'.format( timeout ) )
+        raise RetryableException( Timeout( 'Request Timeout after {0} seconds'.format( timeout ) ) )
 
-      raise ResponseError( 'URLError "{0}" for "{1}" via "{2}"'.format( e, url, self.proxy ) )
+      raise RetryableException( ResponseError( 'URLError "{0}" for "{1}" via "{2}"'.format( e, url, self.proxy ) ), reason=e.reason )
 
     except socket.timeout:
-      raise Timeout( 'Request Timeout after {0} seconds'.format( timeout ) )
+      raise RetryableException( Timeout( 'Request Timeout after {0} seconds'.format( timeout ) ) )
 
-    except socket.error as e:
-      raise ResponseError( 'Socket Error "{0}"'.format( e ) )
+    except OSError as e:
+      raise RetryableException( ResponseError( 'Socket Error "{0}"'.format( e ) ), reason=e.strerror )
 
     http_code = resp.code
     if http_code not in ( 200, 201, 202, 400, 401, 403, 404, 500 ):
@@ -263,20 +323,20 @@ class CInP():
       logging.debug( 'cinp: setting auth info, id "{0}"'.format( auth_id ) )
       self.opener.addheaders += [ ( 'Auth-Id', auth_id ), ( 'Auth-Token', auth_token ) ]
 
-  def describe( self, uri, timeout=30  ):
+  def describe( self, uri, timeout=30, retry_count=0, retry_on=None ):
     """
     DESCRIE
     """
     logging.debug( 'cinp: DESCRIBE "{0}"'.format( uri ) )
-    ( http_code, data, _ ) = self._request( 'DESCRIBE', uri, timeout=timeout )
+    ( http_code, data, header_map ) = self._request( 'DESCRIBE', uri, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 200:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for DESCRIBE'.format( http_code ) )
       raise ResponseError( 'Unexpected HTTP Code "{0}" for DESCRIBE'.format( http_code ) )
 
-    return data
+    return data, header_map[ 'Type' ]
 
-  def list( self, uri, filter_name=None, filter_value_map=None, position=0, count=10, timeout=30 ):
+  def list( self, uri, filter_name=None, filter_value_map=None, position=0, count=10, timeout=30, retry_count=0, retry_on=None ):
     """
     LIST
     """
@@ -294,7 +354,7 @@ class CInP():
       header_map[ 'Filter' ] = filter_name
 
     logging.debug( 'cinp: LIST "{0}" with filter "{1}"'.format( uri, filter_name ) )
-    ( http_code, id_list, header_map ) = self._request( 'LIST', uri, data=filter_value_map, header_map=header_map, timeout=timeout )
+    ( http_code, id_list, header_map ) = self._request( 'LIST', uri, data=filter_value_map, header_map=header_map, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 200:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for LIST'.format( http_code ) )
@@ -313,7 +373,7 @@ class CInP():
 
     return ( id_list, count_map )
 
-  def get( self, uri, force_multi_mode=False, timeout=30 ):
+  def get( self, uri, force_multi_mode=False, timeout=30, retry_count=0, retry_on=None ):
     """
     GET
     """
@@ -322,7 +382,7 @@ class CInP():
       header_map[ 'Multi-Object' ] = True
 
     logging.debug( 'cinp: GET "{0}"'.format( uri ) )
-    ( http_code, rec_values, header_map ) = self._request( 'GET', uri, header_map=header_map, timeout=timeout )
+    ( http_code, rec_values, header_map ) = self._request( 'GET', uri, header_map=header_map, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 200:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for GET'.format( http_code ) )
@@ -334,7 +394,7 @@ class CInP():
 
     return rec_values
 
-  def create( self, uri, values, timeout=30 ):
+  def create( self, uri, values, timeout=30, retry_count=0, retry_on=None ):
     """
     CREATE
     """
@@ -342,7 +402,7 @@ class CInP():
       raise InvalidRequest( 'values must be a dict' )
 
     logging.debug( 'cinp: CREATE "{0}"'.format( uri ) )
-    ( http_code, rec_values, header_map ) = self._request( 'CREATE', uri, data=values, timeout=timeout )
+    ( http_code, rec_values, header_map ) = self._request( 'CREATE', uri, data=values, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 201:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for CREATE'.format( http_code ) )
@@ -359,7 +419,7 @@ class CInP():
 
     return ( object_id, rec_values )
 
-  def update( self, uri, values, force_multi_mode=False, timeout=30 ):
+  def update( self, uri, values, force_multi_mode=False, timeout=30, retry_count=0, retry_on=None ):
     """
     UPDATE
     """
@@ -371,7 +431,7 @@ class CInP():
       header_map[ 'Multi-Object' ] = True
 
     logging.debug( 'cinp: UPDATE "{0}"'.format( uri ) )
-    ( http_code, rec_values, _ ) = self._request( 'UPDATE', uri, data=values, header_map=header_map, timeout=timeout )
+    ( http_code, rec_values, _ ) = self._request( 'UPDATE', uri, data=values, header_map=header_map, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 200:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for UPDATE'.format( http_code ) )
@@ -383,12 +443,12 @@ class CInP():
 
     return rec_values
 
-  def delete( self, uri, timeout=30 ):
+  def delete( self, uri, timeout=30, retry_count=0, retry_on=None ):
     """
     DELETE
     """
     logging.debug( 'cinp: DELETE "{0}"'.format( uri ) )
-    ( http_code, _, _ ) = self._request( 'DELETE', uri, timeout=timeout )
+    ( http_code, _, _ ) = self._request( 'DELETE', uri, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 200:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for DELETE'.format( http_code ) )
@@ -396,7 +456,7 @@ class CInP():
 
     return True
 
-  def call( self, uri, args, force_multi_mode=False, timeout=30 ):
+  def call( self, uri, args, force_multi_mode=False, timeout=30, retry_count=0, retry_on=None ):
     """
     CALL
     """
@@ -408,7 +468,7 @@ class CInP():
       header_map[ 'Multi-Object' ] = True
 
     logging.debug( 'cinp: CALL "{0}"'.format( uri ) )
-    ( http_code, return_value, _ ) = self._request( 'CALL', uri, data=args, header_map=header_map, timeout=timeout )
+    ( http_code, return_value, _ ) = self._request( 'CALL', uri, data=args, header_map=header_map, timeout=timeout, retry_count=retry_count, retry_on=retry_on )
 
     if http_code != 200:
       logging.warning( 'cinp: Unexpected HTTP Code "{0}" for CALL'.format( http_code ) )
@@ -416,7 +476,7 @@ class CInP():
 
     return return_value
 
-  def getMulti( self, uri, id_list=None, chunk_size=10 ):
+  def getMulti( self, uri, id_list=None, chunk_size=10, retry_count=0, retry_on=None ):
     """
     returns a generator that will iterate over the uri/id_list, reterieving from the server in chunk_size blocks
     each item is ( rec_id, rec_values )
@@ -439,7 +499,7 @@ class CInP():
     pos = 0
 
     while pos < len( id_list ):
-      tmp_data = self.get( self.uri.build( namespace, model, None, id_list[ pos: pos + chunk_size ] ), force_multi_mode=True )
+      tmp_data = self.get( self.uri.build( namespace, model, None, id_list[ pos: pos + chunk_size ] ), force_multi_mode=True, retry_count=retry_count, retry_on=retry_on )
       pos += chunk_size
       for key in tmp_data:
         result_list.append( ( key, tmp_data[ key ] ) )
@@ -447,21 +507,21 @@ class CInP():
       while len( result_list ) > 0:
         yield result_list.pop( 0 )
 
-  def getFilteredObjects( self, uri, filter_name=None, filter_value_map=None, list_chunk_size=100, get_chunk_size=10, timeout=30 ):
+  def getFilteredObjects( self, uri, filter_name=None, filter_value_map=None, list_chunk_size=100, get_chunk_size=10, timeout=30, retry_count=0, retry_on=None ):
     pos = 0
     total = 1
     while pos < total:
-      ( tmp_id_list, count_map ) = self.list( uri, filter_name=filter_name, filter_value_map=filter_value_map, position=pos, count=list_chunk_size )
+      ( tmp_id_list, count_map ) = self.list( uri, filter_name=filter_name, filter_value_map=filter_value_map, position=pos, count=list_chunk_size, retry_count=retry_count, retry_on=retry_on )
       id_list = self.uri.extractIds( tmp_id_list )
       pos = count_map[ 'position' ] + count_map[ 'count' ]
       total = count_map[ 'total' ]
-      return self.getMulti( uri, id_list, get_chunk_size )  # TODO: need to found out how to get the next chunk, return/yeild something
+      return self.getMulti( uri, id_list, get_chunk_size, retry_count=retry_count, retry_on=retry_on )  # TODO: need to found out how to get the next chunk, return/yeild something
 
-  def getFilteredURIs( self, uri, filter_name=None, filter_value_map=None, list_chunk_size=100, get_chunk_size=10, timeout=30 ):
+  def getFilteredURIs( self, uri, filter_name=None, filter_value_map=None, list_chunk_size=100, get_chunk_size=10, timeout=30, retry_count=0, retry_on=None ):
     pos = 0
     total = 1
     while pos < total:
-      ( tmp_id_list, count_map ) = self.list( uri, filter_name=filter_name, filter_value_map=filter_value_map, position=pos, count=list_chunk_size )
+      ( tmp_id_list, count_map ) = self.list( uri, filter_name=filter_name, filter_value_map=filter_value_map, position=pos, count=list_chunk_size, retry_count=retry_count, retry_on=retry_on )
       id_list = self.uri.extractIds( tmp_id_list )
       pos = count_map[ 'position' ] + count_map[ 'count' ]
       total = count_map[ 'total' ]
